@@ -303,7 +303,7 @@ eog_plugin_engine_load_dir (const gchar *dir)
 	d = g_dir_open (dir, 0, &error);
 
 	if (!d)	{
-		g_warning (error->message);
+		g_warning ("%s", error->message);
 		g_error_free (error);
 
 		return;
